<?php
namespace poony_plugin;

/**
 * Setting class Poony
 */


class NewOrderPoony
{

    function __construct()
    {
        add_action( 'woocommerce_order_status_processing', array($this,'add_order_to_poony'), 10, 3 );
        add_action('activated_plugin', array($this,'my_save_error'));
    }

    public function my_save_error(){
        file_put_contents(POONY_PLUGIN_PATH.'/error_activation.txt', ob_get_contents());
    }

    public function add_order_to_poony($order_id){

        $locationField = get_option('poony_location_index_field');

        $location_index = 1;

        if (!is_null($locationField) && strlen ( $locationField) > 0){
            $location_index = get_post_meta( $order_id, $locationField, true );
        }

        $poonyUsr = get_option('poony_usr_pickup_'.$location_index);
        $poonyPw = get_option('poony_pw_pickup_'.$location_index);
        $poonyBaseUrl = get_option('poony_base_url_pickup_'.$location_index);

        if (!$order_id || !$poonyUsr || !$poonyPw || !$poonyBaseUrl || !$location_index) {
            return;
        }

        $this->authUserInPoony($poonyBaseUrl, $poonyUsr, $poonyPw, $order_id, $location_index);

    }

    public function authUserInPoony($poonyBaseUrl, $poonyUsr, $poonyPw, $order_id, $location_index){
        $loginUrl = $poonyBaseUrl."/credential/signIn";
        $postdata = array(
            'username' => $poonyUsr,
            'password' => $poonyPw
        );


        for ($i = 0; $i < 2; $i++) {
            $response = $this->authUser($loginUrl, $postdata, $location_index);
            $user = json_decode($response);
            if ($user && $user->result && $user->result->access_token){
                $this->addOrderToPoony($poonyBaseUrl, $order_id, $user->result->access_token, $location_index);
                break;
            }
        }

    }

    public function authUser($loginUrl, $postdata, $location_index){
        $pluginlog = POONY_PLUGIN_PATH.'debug.log';
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $loginUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode($postdata),
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));

        $response = curl_exec($curl);

        $user = json_decode($response);
        if(!($user && $user->result && $user->result->access_token)){
            $timeZoneName = get_option('poony_time_zone_name_pickup_'.$location_index) !== null
                ? get_option('poony_time_zone_name_pickup_'.$location_index) : null;
            $now = new \DateTime();
            $now->setTimezone(new \DateTimeZone($timeZoneName));
            error_log($now->format('Y-m-d H:i:s')."-ERROR AUTH: ".json_encode($postdata)."\n", 3, $pluginlog);
            error_log("response: ".$response."\n", 3, $pluginlog);
            error_log("user: ".$user."\n", 3, $pluginlog);
        }

        curl_close($curl);

        return $response;
    }

    public function addOrderToPoony($poonyBaseUrl, $order_id, $access_token, $location_index){

        $createOrderUrl = $poonyBaseUrl."/api/order";

        $order = wc_get_order($order_id);

        $items = array();

        $locationField = get_option('poony_location_index_field');

        foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
            $shipping_item_obj_data = $shipping_item_obj->get_data();
            $shipping_method_id          = $shipping_item_obj->get_method_id(); // The method ID
            $shipping_data_total        = $shipping_item_obj_data['total'];

            $shipping_data_total_order = null;
            if ($shipping_data_total != null){
                $shipping_data_total_order = (object) array(
                    'code' => 'EUR',
                    'value' => $shipping_data_total
                );
            }
        }

        if ($shipping_method_id == "local_pickup"){
            $this->addReservationToPoony($poonyBaseUrl, $order_id, $access_token, $location_index);
            return;
        }

        // Iterating through each WC_Order_Item_Product objects
        foreach ($order->get_items() as $item_key => $item ) {

            ## Access Order Items data properties (in an array of values) ##
            $item_data = $item->get_data();

            // Get data from The WC_product object using methods (examples)
            $product = $item->get_product(); // Get the WC_Product object

            $all_meta = $item->get_meta_data();

            $subItems = array();
            if( $all_meta ) {
                foreach( $all_meta as $meta ) {
                    $meta_key = $meta->key;
                    $meta_value = $meta->value;
                    if ($meta_key !== 'product_extras' && $meta_value){
                        $meta_description = str_replace("&euro;", "EUR ", strip_tags($meta_value));
                        $meta_description = $meta_description."|";
                        $needle = "|";

                        while (strpos($meta_description, $needle) !== false) {
                            $needlePosition = strpos($meta_description, '|');
                            $meta_description2 = substr($meta_description, 0, $needlePosition);
                            if (strpos($meta_description2, '(') !== false){
                                $meta_description_final = substr($meta_description2, 0, strpos($meta_description2, '('));
                            } else {
                                $meta_description_final = $meta_description2;
                            }
                            if (strlen ($meta_description_final) > 1) {
                                array_push($subItems, (object)array(
                                    'code' => null,
                                    'description' => $meta_description_final,
                                    'qty' => 1,
                                    'unitPrice' => null
                                ));
                            }
                            $meta_description = substr($meta_description, $needlePosition+1);
                        }

                    }

                }
            }

            if (strpos($item_data['name'], '<') !== false){
                $descriptionFinal = substr($item_data['name'], 0, strpos($item_data['name'], '<'));
            }else {
                $descriptionFinal = $item_data['name'];
            }
            array_push($items, (object) array(
                'code' => $item_data['product_id'].'-'.$item_data['variation_id'],
                'description' => $descriptionFinal,
                'qty' => $item_data['quantity'],
                'subItems' => $subItems,
                'unitPrice' => (object) array(
                    'code' => 'EUR',
                    'value' => $item_data['total']
                )));
        }

        $order_data = $order->get_data();
        $pluginlog = POONY_PLUGIN_PATH.'debug.log';

        // Get order total
        $order_total = is_callable(array($order, 'get_total')) ? $order->get_total() : $order->order_total;

        $order_total_order = null;
        if ($order_total != null){
            $order_total_order = (object) array(
                'code' => 'EUR',
                'value' => $order_total
            );
        }

        // Get order subtotal
        $order_subtotal = $order->get_subtotal();
        // Get the correct number format (2 decimals)
        $order_subtotal = number_format( $order_subtotal, 2 );

        $order_subtotal_order = null;
        if ($order_subtotal != null){
            $order_subtotal_order = (object) array(
                'code' => 'EUR',
                'value' => $order_subtotal
            );
        }

        $deliveryDateField = get_option('poony_delivery_date_field');
        $deliveryTimeField = get_option('poony_delivery_time_field');
        $deliveryNoteField = get_option('poony_delivery_note_field');

        $scheduled_date = get_post_meta( $order_id, $deliveryDateField, true );
        $scheduled_time = null;

        $delivery_time = get_post_meta( $order_id, $deliveryTimeField, true );

        if (!is_null($delivery_time) && strlen ($delivery_time) > 0){
            $num = substr($delivery_time, 0, 2);
            $int = (int)$num;
            if (($num !== 12 && strpos($delivery_time, 'PM') !== false) || ($num === 12 && strpos($delivery_time, 'AM') !== false)){
                $int = $int + 12;
                $scheduled_time = $int.substr($delivery_time, 2, 3);
            } else {
                $scheduled_time = substr($delivery_time, 0, 5);
            }
        }


        if (is_null($scheduled_date) || strlen ($scheduled_date) == 0){
            $scheduled_date = null;
        }
        if (is_null($scheduled_time) || strlen ($scheduled_time) == 0){
            $scheduled_time = null;
        }

        $placeFirstPickup = (object) array(
            'pointType' => 'STARTING_PICKUP',
            'acronym' => get_option( 'poony_store_name_pickup_'.$location_index),
            'address' => (object) array(
                'careOf' => get_option( 'poony_store_care_of_pickup_'.$location_index),
                'city' => get_option( 'poony_store_city_pickup_'.$location_index),
                'country' => get_option( 'poony_store_country_pickup_'.$location_index),
                'postalCode' => get_option( 'poony_store_postal_code_pickup_'.$location_index),
                'province' => get_option( 'poony_store_province_pickup_'.$location_index),
                'street' => get_option( 'poony_store_address_pickup_'.$location_index)
            ),
            'amount' => null,
            'arrival' => false,
            'delivery' => false,
            'note' => '',
            'numberOfPackages' => 1,
            'payment' => null,
            'phoneNumber' => get_option( 'poony_store_phone_pickup_'.$location_index),
            'pickup' => true,
            'position' => 0,
            'starting' => true
        );
        $phoneNumber = $order_data['billing']['phone'];
        if (strlen ( $phoneNumber) < 10){
            $phoneNumber = $phoneNumber.'0000000000';
            $phoneNumber = substr($phoneNumber,0,10);
        }

        $deliveryNote = get_user_meta( get_current_user_id(), $deliveryNoteField, true );

        $placeLastDelivery = (object) array(
            'pointType' => 'ARRIVAL_DELIVERY',
            'acronym' => $order_data['shipping'] && $order_data['shipping']['company']
                ? $order_data['shipping']['company'] : $order_data['billing']['first_name'].' '.$order_data['billing']['last_name'],
            'address' => (object) array(
                'careOf' => $order_data['billing']['first_name'].' '.$order_data['billing']['last_name'],
                'city' => $order_data['shipping']['city'],
                'country' => $order_data['shipping']['country'],
                'postalCode' => $order_data['shipping']['postcode'],
                'province' => $order_data['shipping']['state'],
                'street' => $order_data['shipping']['address_1']
            ),
            'amount' => strcmp($order_data['payment_method'],'cod') == 0
                ? (object) array(
                    'code' => $order_data['currency'],
                    'value' => $order_data['total']
                ) : null,
            'arrival' => true,
            'delivery' => true,
            'note' => $deliveryNote !== null ? $deliveryNote : null,
            'numberOfPackages' => 1,
            'payment' => strcmp($order_data['payment_method'],'cod') == 0
                ? (object) array(
                    'done' => false,
                    'type' => 'CASH_ON_DELIVERY'
                ) : null,
            'phoneNumber' => $phoneNumber,
            'pickup' => false,
            'position' => 999,
            'starting' => false
        );

        $timeZoneName = get_option('poony_time_zone_name_pickup_'.$location_index) !== null ? get_option('poony_time_zone_name_pickup_'.$location_index) : null;

        if (!is_null($scheduled_date) && !is_null($scheduled_time)){
            $scheduleAt = $scheduled_date." ".substr($scheduled_time, 0, 5).":00";
            // $scheduleAt = new \DateTime($scheduled_date, new \DateTimeZone($timeZoneName));
        } else {
            $scheduleAt = null;
        }

        if ($scheduleAt != null){
            $now = new \DateTime();
            $now->setTimezone(new \DateTimeZone($timeZoneName));
            $scheduledOn = new \DateTime($scheduleAt, new \DateTimeZone($timeZoneName));
            $scheduledOn->modify('-15 minutes');
            $differenceMinutes = round(abs($scheduledOn->getTimestamp() - $now->getTimestamp()) / 60);
            if ($differenceMinutes < 32){
                $scheduleAt = null;
            } else {
                $scheduleAt = $scheduledOn->format('c');
            }
        }

        $orderPayment = array(
            'type' => 'WALLET'
        );

        $orderAutoAssignment = get_option('poony_auto_assignment_pickup_'.$location_index) == 1;
        $orderSendNotification = get_option('poony_send_notification_pickup_'.$location_index) == 1;

        $toPrint = get_option('poony_to_print_pickup_'.$location_index) == 1;
        $locationCode = get_option('poony_location_code_pickup_'.$location_index);

        $postdata = array(
            'timeZoneName' => $timeZoneName !== null ? $timeZoneName : null,
            'itemsNote' => $order_data['customer_note'] ? $order_data['customer_note'] : null,
            'itemsSubTotal' => $order_subtotal_order,
            'itemsDelivery' => $shipping_data_total_order,
            'itemsTot' => $order_total_order,
            'rideType' => 'ONE_TO_ONE',
            'numberOfPackages' => 1,
            'scheduled' => !is_null($scheduleAt),
            'scheduleAt' => $scheduleAt,
            'payment' => (object) $orderPayment,
            'places' => array(
                $placeLastDelivery, $placeFirstPickup
            ),
            'items'=>$items,
            'toPrint'=> $toPrint,
            'locationCode'=> $locationCode,
            'autoAssignment' => $orderAutoAssignment,
            'sendNotification' => $orderSendNotification,
            'description' => 'Woocommerce Order n. '.$order_data['number'],
            'dimension' =>(object) array(
                'depth'=>(object) array(
                    'symbol'=>'mm',
                    'value'=> 100
                ),
                'height'=>(object) array(
                    'symbol'=>'mm',
                    'value'=> 300
                ),
                'width'=>(object) array(
                    'symbol'=>'mm',
                    'value'=> 200
                )
            ),
            'weight'=>(object) array(
                'quantity'=>(object) array(
                    'symbol'=>'kg',
                    'value'=> 3
                )
            )
        );

        for ($i = 0; $i < 2; $i++) {
            $response = $this->createOrder($access_token, $postdata, $createOrderUrl, $location_index);
            $order = json_decode($response);
            if ($order && $order->result){
                break;
            }
        }

        return $response;

    }

    public function createOrder($access_token, $postdata, $createOrderUrl, $location_index){
        $pluginlog = POONY_PLUGIN_PATH.'debug.log';

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => $createOrderUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode($postdata),
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json",
                "Authorization: Bearer ".$access_token
            ),
        ));
        $response = curl_exec($curl);

        $order = json_decode($response);
        if(!($order && $order->result)){
            $timeZoneName = get_option('poony_time_zone_name_pickup_'.$location_index) !== null ? get_option('poony_time_zone_name_pickup_'.$location_index) : null;
            $now = new \DateTime();
            $now->setTimezone(new \DateTimeZone($timeZoneName));
            error_log($now->format('Y-m-d H:i:s')."_ERROR ORDER: ".json_encode($postdata)."\n", 3, $pluginlog);
            error_log("response: ".$response."\n", 3, $pluginlog);
        }

        curl_close($curl);

        return $response;
    }

    public function addReservationToPoony($poonyBaseUrl, $order_id, $access_token, $location_index){

        $createReservationUrl = $poonyBaseUrl."/api/reservation";

        $order = wc_get_order($order_id);

        $items = array();

        $locationField = get_option('poony_location_index_field');

        foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
            $shipping_item_obj_data = $shipping_item_obj->get_data();
            $shipping_method_id          = $shipping_item_obj->get_method_id(); // The method ID
            $shipping_data_total        = $shipping_item_obj_data['total'];

            $shipping_data_total_order = null;
            if ($shipping_data_total != null){
                $shipping_data_total_order = (object) array(
                    'code' => 'EUR',
                    'value' => $shipping_data_total
                );
            }
        }

        if ($shipping_method_id != "local_pickup"){
            return;
        }

        // Iterating through each WC_Order_Item_Product objects
        foreach ($order->get_items() as $item_key => $item ) {

            ## Access Order Items data properties (in an array of values) ##
            $item_data = $item->get_data();

            // Get data from The WC_product object using methods (examples)
            $product = $item->get_product(); // Get the WC_Product object

            $all_meta = $item->get_meta_data();

            $subItems = array();
            if( $all_meta ) {
                foreach( $all_meta as $meta ) {
                    $meta_key = $meta->key;
                    $meta_value = $meta->value;
                    if ($meta_key !== 'product_extras' && $meta_value){
                        $meta_description = str_replace("&euro;", "EUR ", strip_tags($meta_value));
                        $meta_description = $meta_description."|";
                        $needle = "|";

                        while (strpos($meta_description, $needle) !== false) {
                            $needlePosition = strpos($meta_description, '|');
                            $meta_description2 = substr($meta_description, 0, $needlePosition);
                            if (strpos($meta_description2, '(') !== false){
                                $meta_description_final = substr($meta_description2, 0, strpos($meta_description2, '('));
                            } else {
                                $meta_description_final = $meta_description2;
                            }
                            if (strlen ($meta_description_final) > 1) {
                                array_push($subItems, (object)array(
                                    'code' => null,
                                    'description' => $meta_description_final,
                                    'qty' => 1,
                                    'unitPrice' => null
                                ));
                            }
                            $meta_description = substr($meta_description, $needlePosition+1);
                        }

                    }

                }
            }

            if (strpos($item_data['name'], '<') !== false){
                $descriptionFinal = substr($item_data['name'], 0, strpos($item_data['name'], '<'));
            }else {
                $descriptionFinal = $item_data['name'];
            }
            array_push($items, (object) array(
                'code' => $item_data['product_id'].'-'.$item_data['variation_id'],
                'description' => $descriptionFinal,
                'qty' => $item_data['quantity'],
                'subItems' => $subItems,
                'unitPrice' => (object) array(
                    'code' => 'EUR',
                    'value' => $item_data['total']
                )));
        }

        $order_data = $order->get_data();

        // Get order total
        $order_total = is_callable(array($order, 'get_total')) ? $order->get_total() : $order->order_total;

        $order_total_order = null;
        if ($order_total != null){
            $order_total_order = (object) array(
                'code' => 'EUR',
                'value' => $order_total
            );
        }

        // Get order subtotal
        $order_subtotal = $order->get_subtotal();
        // Get the correct number format (2 decimals)
        $order_subtotal = number_format( $order_subtotal, 2 );

        $order_subtotal_order = null;
        if ($order_subtotal != null){
            $order_subtotal_order = (object) array(
                'code' => 'EUR',
                'value' => $order_subtotal
            );
        }

        $deliveryDateField = get_option('poony_delivery_date_field');
        $deliveryTimeField = get_option('poony_delivery_time_field');
        $deliveryNoteField = get_option('poony_delivery_note_field');

        $scheduled_date = get_post_meta( $order_id, $deliveryDateField, true );
        $scheduled_time = null;

        $delivery_time = get_post_meta( $order_id, $deliveryTimeField, true );

        if (!is_null($delivery_time) && strlen ($delivery_time) > 0){
            $num = substr($delivery_time, 0, 2);
            $int = (int)$num;
            if (($num !== 12 && strpos($delivery_time, 'PM') !== false) || ($num === 12 && strpos($delivery_time, 'AM') !== false)){
                $int = $int + 12;
                $scheduled_time = $int.substr($delivery_time, 2, 3);
            } else {
                $scheduled_time = substr($delivery_time, 0, 5);
            }
        }

        if (is_null($scheduled_date) || strlen ($scheduled_date) == 0){
            $scheduled_date = null;
        }
        if (is_null($scheduled_time) || strlen ($scheduled_time) == 0){
            $scheduled_time = null;
        }

        $placeFirstPickup = (object) array(
            'pointType' => 'STARTING_PICKUP',
            'acronym' => get_option( 'poony_store_name_pickup_'.$location_index),
            'address' => (object) array(
                'careOf' => get_option( 'poony_store_care_of_pickup_'.$location_index),
                'city' => get_option( 'poony_store_city_pickup_'.$location_index),
                'country' => get_option( 'poony_store_country_pickup_'.$location_index),
                'postalCode' => get_option( 'poony_store_postal_code_pickup_'.$location_index),
                'province' => get_option( 'poony_store_province_pickup_'.$location_index),
                'street' => get_option( 'poony_store_address_pickup_'.$location_index)
            ),
            'amount' => null,
            'arrival' => false,
            'delivery' => false,
            'note' => '',
            'numberOfPackages' => 1,
            'payment' => null,
            'phoneNumber' => get_option( 'poony_store_phone_pickup_'.$location_index),
            'pickup' => true,
            'position' => 0,
            'starting' => true
        );
        $phoneNumber = $order_data['billing']['phone'];
        if (strlen ( $phoneNumber) < 10){
            $phoneNumber = $phoneNumber.'0000000000';
            $phoneNumber = substr($phoneNumber,0,10);
        }

        $timeZoneName = get_option('poony_time_zone_name_pickup_'.$location_index) !== null ? get_option('poony_time_zone_name_pickup_'.$location_index) : null;

        if (!is_null($scheduled_date) && !is_null($scheduled_time)){
            $scheduleAt = $scheduled_date." ".substr($scheduled_time, 0, 5).":00";
            // $scheduleAt = new \DateTime($scheduled_date, new \DateTimeZone($timeZoneName));
        } else {
            $scheduleAt = null;
        }

        if ($scheduleAt != null){
            $now = new \DateTime();
            $now->setTimezone(new \DateTimeZone($timeZoneName));
            $scheduledOn = new \DateTime($scheduleAt, new \DateTimeZone($timeZoneName));
            $scheduleAt = $scheduledOn->format('c');
        }

        $orderPayment = array(
            'type' => 'WALLET'
        );

        $toPrint = get_option('poony_to_print_pickup_'.$location_index) == 1;
        $locationCode = get_option('poony_location_code_pickup_'.$location_index);

        $deliveryNote = get_user_meta( get_current_user_id(), $deliveryNoteField, true );

        $postdata = array(
            'timeZoneName' => $timeZoneName !== null ? $timeZoneName : null,
            'itemsNote' => $order_data['customer_note'] ? $order_data['customer_note'] : null,
            'itemsSubTotal' => $order_subtotal_order,
            'itemsDelivery' => $shipping_data_total_order,
            'itemsTot' => $order_total_order,
            'numberOfPackages' => 1,
            'scheduleAt' => $scheduleAt,
            'amount' => strcmp($order_data['payment_method'],'cod') == 0
                ? (object) array(
                    'code' => $order_data['currency'],
                    'value' => $order_data['total']
                ) : null,
            'payment' => strcmp($order_data['payment_method'],'cod') == 0
                ? (object) array(
                    'done' => false,
                    'type' => 'CASH_ON_DELIVERY'
                ) : null,
            'from' => $placeFirstPickup,
            'items'=>$items,
            'toPrint'=> $toPrint,
            'locationCode'=> $locationCode,
            'description' => 'Woocommerce Order n. '.$order_data['number'],
            'careOf' => $order_data['billing']['first_name'].' '.$order_data['billing']['last_name'],
            'phoneNumber' => $phoneNumber,
            'note' => $deliveryNote !== null ? $deliveryNote : null
        );

        for ($i = 0; $i < 2; $i++) {
            $response = $this->createReservation($access_token, $postdata, $createReservationUrl, $location_index);
            $reservation = json_decode($response);
            if ($reservation && $reservation->id){
                break;
            }
        }

        return $response;

    }

    public function createReservation($access_token, $postdata, $createReservationUrl, $location_index){
        $pluginlog = POONY_PLUGIN_PATH.'debug.log';

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => $createReservationUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode($postdata),
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json",
                "Authorization: Bearer ".$access_token
            ),
        ));
        $response = curl_exec($curl);

        $order = json_decode($response);
        if(!($order && $order->result)){
            $timeZoneName = get_option('poony_time_zone_name_pickup_'.$location_index) !== null ? get_option('poony_time_zone_name_pickup_'.$location_index) : null;
            $now = new \DateTime();
            $now->setTimezone(new \DateTimeZone($timeZoneName));
            error_log($now->format('Y-m-d H:i:s')."_ERROR RESERVATION: ".json_encode($postdata)."\n", 3, $pluginlog);
            error_log("response: ".$response."\n", 3, $pluginlog);
        }

        curl_close($curl);

        return $response;
    }
}